/***************************************************************************
                          sqmixer.cpp  -  description
                             -------------------
    begin                : Sat Apr  6 22:43:57 CEST 2002
    copyright            : (C) 2002 by Sven Queisser
    email                : svenqueisser@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qhbox.h>
#include <qaccel.h>
#include <qtimer.h>

#include <sys/soundcard.h>

#include "sqmixer.h"
#include "sqmixerhw.h"
#include "settings.h"


extern const char *channelnames[];

extern smixer *mixer;

SQMixerApp::SQMixerApp() : QMainWindow()
{
    int i;
    useVol = useRec = useMute = configIsValid = false;
    QString title("SQMixer " VERSION " for ");

    /* array of channels and its view-qactions (probably not all will be used)*/
    channels = new mchannel*[SOUND_MIXER_NRDEVICES];
    viewchannels = new QAction*[SOUND_MIXER_NRDEVICES];

    title.append(mixer->info.name);
    setCaption(title);

    initActions();
    initMenuBar();

    QHBox *box = new QHBox(this);
    setCentralWidget(box);

    box->setSpacing(5);
    box->setMargin(5);

    QSettings *settings = new QSettings;
    /* read viewMask on startup. use all devices if no config! */
//    viewMask = settings->readNumEntry("/SQMixer/viewMask", mixer->devs);
    delete settings;

    /* create widget for each channel: */
    for(i = 0; i < SOUND_MIXER_NRDEVICES; i++){
        if(mixer->devs & (1 << i)){     /* if channel exists... */
            channels[i] = new mchannel(box, i);
            viewchannels[i] = new QAction(channelnames[i], channelnames[i], 0, this, 0, true);
            connect(viewchannels[i], SIGNAL(toggled(bool)), channels[i], SLOT(enable(bool)));

            viewchannels[i]->addTo(viewMenu);
//            if(viewMask & (1 << i))
                viewchannels[i]->setOn(1);
//            else
//                channels[i]->enable(false);


            connect(this, SIGNAL(allChanged()), channels[i], SLOT(updateAll() ) );
            connect(this, SIGNAL(recsChanged()), channels[i], SLOT(updateRecs() ) );
            connect(channels[i], SIGNAL(recsChanged()), this, SIGNAL(recsChanged() ) );
        }
    }

    /* timer for permanent updates: */
    counter = new QTimer( this );
    connect(counter, SIGNAL( timeout() ), SIGNAL( allChanged() ) );
    autoUpdate->setOn( true );
    counter->start(100);

    loadConf();
}

SQMixerApp::~SQMixerApp()
{

}

/** initializes all QActions of the application */
void SQMixerApp::initActions(){

  fileQuit = new QAction("Exit", "E&xit", QAccel::stringToKey("Ctrl+Q"), this);
  connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));

  helpAboutApp = new QAction("About", "&About...", 0, this);
  connect(helpAboutApp, SIGNAL(activated()), SLOT(slotHelpAbout()));

  autoUpdate = new QAction("auto update", "&auto update", 0, this);
  connect(autoUpdate, SIGNAL(toggled(bool)), SLOT(setTimer(bool)));

  menuSaveConf = new QAction("save config", "&save configuration", 0, this);
  connect(menuSaveConf, SIGNAL(activated()), SLOT(saveConf()) );

  menuLoadConf = new QAction("load config", "&load configuration", 0, this);
  connect(menuLoadConf, SIGNAL(activated()), SLOT(loadConf()) );

  menuUseVol = new QAction("use vol", "use &volume settings from config file", 0, this);
  connect(menuUseVol, SIGNAL(toggled(bool)), SLOT(setUseVol(bool)) );

  menuUseRec = new QAction("use rec", "use &recording settings from config file", 0, this);
  connect(menuUseRec, SIGNAL(toggled(bool)), SLOT(setUseRec(bool)) );

  menuUseMute = new QAction("use mute", "use &muting settings from config file", 0, this);
  connect(menuUseMute, SIGNAL(toggled(bool)), SLOT(setUseMute(bool)) );
}

void SQMixerApp::initMenuBar()
{
  fileMenu=new QPopupMenu();
  fileQuit->addTo(fileMenu);

  viewMenu=new QPopupMenu();

  helpMenu=new QPopupMenu();
  helpAboutApp->addTo(helpMenu);

  settingsMenu = new QPopupMenu();
  autoUpdate->addTo(settingsMenu);
  autoUpdate->setToggleAction( true );

  menuLoadConf->addTo(settingsMenu);

  menuSaveConf->addTo(settingsMenu);

  menuUseVol->addTo(settingsMenu);
  menuUseVol->setToggleAction( true );

  menuUseMute->addTo(settingsMenu);
  menuUseMute->setToggleAction( true );

  menuUseRec->addTo(settingsMenu);
  menuUseRec->setToggleAction( true );


  menuBar()->insertItem("&File", fileMenu);
  menuBar()->insertItem("&View", viewMenu);
  menuBar()->insertItem("&Settings", settingsMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem("&Help", helpMenu);
}

void SQMixerApp::slotFileQuit()
{
    qApp->quit();
}

void SQMixerApp::slotHelpAbout()
{
    QMessageBox::about(this,"About...", "SQMixer\nVersion " VERSION "\n(c) 2002 by Sven Queisser" );
}

void SQMixerApp::setTimer( bool enable ){
    if(enable)
        counter->start(100);
    else
        counter->stop();
}

void SQMixerApp::saveConf(){
    int i;
    QString *confBranch = new QString;

    QSettings *settings = new QSettings;
    for(i = 0; i < SOUND_MIXER_NRDEVICES; i++){
        if((1 << i) & mixer->devs){

        /* write name of channel */
            confBranch->sprintf("/SQMixer/Channel%d/ChannelName", i);
            settings->writeEntry( *confBranch , channels[i]->getName() );

        /* write muting state */
            confBranch->sprintf("/SQMixer/Channel%d/Mute", i);
            settings->writeEntry( *confBranch , channels[i]->getMuteState() );

        /* write recording state */
            confBranch->sprintf("/SQMixer/Channel%d/Rec", i);
            settings->writeEntry( *confBranch , channels[i]->getRecState() );

        /* write locking state */
            confBranch->sprintf("/SQMixer/Channel%d/Lock", i);
            settings->writeEntry( *confBranch , channels[i]->getLockState() );

        /* write display state */
            confBranch->sprintf("/SQMixer/Channel%d/Display", i);
            settings->writeEntry( *confBranch , viewchannels[i]->isOn() );

        /* write volume settings: */
            confBranch->sprintf("/SQMixer/Channel%d/LVol", i);
            settings->writeEntry( *confBranch , channels[i]->getLVolume() );

            confBranch->sprintf("/SQMixer/Channel%d/RVol", i);
            settings->writeEntry( *confBranch , channels[i]->getRVolume() );

            confBranch->sprintf("/SQMixer/Channel%d/LMuteVol", i);
            settings->writeEntry( *confBranch , channels[i]->getLMuteVol() );

            confBranch->sprintf("/SQMixer/Channel%d/RMuteVol", i);
            settings->writeEntry( *confBranch , channels[i]->getRMuteVol() );

        }

    }

    /* General-section in config file: */
    /* include mixer name: */
    confBranch->sprintf("/SQMixer/General/MixerName");
    settings->writeEntry( *confBranch , mixer->info.name);

    /* auto-update-setting */
    confBranch->sprintf("/SQMixer/General/AutoUpdate");
    settings->writeEntry( *confBranch , autoUpdate->isOn());

    /* use volume settings from config file: */
    confBranch->sprintf("/SQMixer/General/UseVol");
    settings->writeEntry( *confBranch , useVol);

    /* use recording source settings from config file: */
    confBranch->sprintf("/SQMixer/General/UseRec");
    settings->writeEntry( *confBranch , useRec);

    /* use muted volume settings from config file: */
    confBranch->sprintf("/SQMixer/General/UseMute");
    settings->writeEntry( *confBranch , useMute);

    /* store settings: */
    delete settings;
    delete confBranch;

}


void SQMixerApp::loadConf(){
    int i;
    QSettings *settings = new QSettings;
    QString *confBranch = new QString;
    QString *mixerName = new QString;
    QString *channelName = new QString;

    /* validate mixer name
     * dont use config if different */
    confBranch->sprintf("/SQMixer/General/MixerName");
    mixerName->append( settings->readEntry( *confBranch ) );

    if(mixerName->compare(mixer->info.name) == 0){
        configIsValid = 1;
        /* auto-update settings: */
        confBranch->sprintf("/SQMixer/General/AutoUpdate");
        autoUpdate->setOn(settings->readBoolEntry( *confBranch, true ));

        /* use volumes from file? */
        confBranch->sprintf("/SQMixer/General/UseVol");
        menuUseVol->setOn(settings->readBoolEntry( *confBranch, false ));

        /* use rec sources from file? */
        confBranch->sprintf("/SQMixer/General/UseRec");
        menuUseRec->setOn(settings->readBoolEntry( *confBranch, false ));

        /* use muting settings from file? */
        confBranch->sprintf("/SQMixer/General/UseMute");
        menuUseMute->setOn(settings->readBoolEntry( *confBranch, false ));

        for(i = 0; i < SOUND_MIXER_NRDEVICES; i++){
            if((1 << i) & mixer->devs){
                confBranch->sprintf("/SQMixer/Channel%d/ChannelName", i);
                channelName->truncate(0);
                channelName->append( settings->readEntry( *confBranch, NULL ) );
                channels[i]->setName(  channelName );

                /* get muting state if desired: */
                confBranch->sprintf("/SQMixer/Channel%d/Mute", i);
                if(useMute)
                    channels[i]->setMuteState(settings->readBoolEntry( *confBranch, channels[i]->getMuteState() ) );

                /* get recording state if desired: */
                confBranch->sprintf("/SQMixer/Channel%d/Rec", i);
                if(useRec)
                    channels[i]->setRecState(settings->readBoolEntry( *confBranch, channels[i]->getRecState() ) );

                /* get locking state (in any case): */
                confBranch->sprintf("/SQMixer/Channel%d/Lock", i);
                channels[i]->setLockState(settings->readBoolEntry( *confBranch, channels[i]->getLockState() ) );

                /* load all the stored volumes */
                confBranch->sprintf("/SQMixer/Channel%d/LMuteVol", i);
                channels[i]->setLMuteVol(settings->readNumEntry( *confBranch, channels[i]->getLMuteVol() ) );

                confBranch->sprintf("/SQMixer/Channel%d/RMuteVol", i);
                channels[i]->setRMuteVol(settings->readNumEntry( *confBranch, channels[i]->getRMuteVol() ) );

                if(useVol){
                    confBranch->sprintf("/SQMixer/Channel%d/LVol", i);
                    channels[i]->setLVolume(settings->readNumEntry( *confBranch, channels[i]->getLVolume() ) );

                    confBranch->sprintf("/SQMixer/Channel%d/RVol", i);
                    channels[i]->setRVolume(settings->readNumEntry( *confBranch, channels[i]->getRVolume() ) );
                }

                /* check if we want to see this channel: */
                confBranch->sprintf("/SQMixer/Channel%d/Display", i);
                viewchannels[i]->setOn(settings->readBoolEntry( *confBranch, viewchannels[i]->isOn() ) );
            }
        }
    }

    delete settings;
    delete confBranch;
    delete mixerName;
    delete channelName;
}


void SQMixerApp :: setUseMute( bool val ){
    useMute = val;
}

void SQMixerApp :: setUseRec( bool val ){
    useRec = val;
}

void SQMixerApp :: setUseVol( bool val ){
    useVol = val;
}
