/***************************************************************************
                          sqmixerhw.cpp  -  description
                             -------------------
    begin                : Tue Apr 9 2002
    copyright            : (C) 2002 by Sven Queisser
    email                : svenqueisser@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* QT headers: */
//#include <qt.h>
#include <qobject.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qslider.h>
#include <qgroupbox.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qcheckbox.h>
#include <qstring.h>

/* file/ioctl: */
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>

/* soundcard: */
#include <sys/soundcard.h>

/* own: */
#include "sqmixerhw.h"
#include "settings.h"

extern smixer *mixer;

/* get channelnames from soundcard.h */
const char *channelnames[] = SOUND_DEVICE_LABELS;

mchannel :: mchannel(QWidget *parent, int SetChannel) : QHBox(parent)
{
    channel = SetChannel; /* channel is valid inside the whole class, so dont change this construct */
    IsStereo = false;
    IsRecDev = false;
    lmute = rmute = 0;

    if(channel < 0 || channel >= SOUND_MIXER_NRDEVICES){
        fprintf(stderr, "mchannel(): I need to know, which channel i am (I cant be %d)!\n", channel);
        return;
    }

    /* check if i am stereo */
    if(mixer->st &(1<<channel))
        IsStereo = 1;

    /* check if i can record */
    if(mixer->rec & (1<<channel))
        IsRecDev = 1;

    lvol = GetLVal();
    rvol = GetRVal();

/* start drawing widgets: */

    this->setFrameStyle(QFrame::Box | QFrame::Sunken);
    this->setMargin(2);

    QVBox *box = new QVBox(this, channelnames[channel]);

    /* label channel: */
    channelLabel = new QLineEdit(channelnames[channel], box);
    channelLabel->setFrame( false );
    channelLabel->setPaletteBackgroundColor( this->paletteBackgroundColor() );
    channelLabel->setMaximumWidth( 65 );

    /* create border for sliders: */
    QHBox *sliderbox = new QHBox(box);
    QVBox *vbox;

    vbox = new QVBox(sliderbox);
    QLabel *ldisp = new QLabel(vbox);
    ldisp->setAlignment(QLabel::AlignHCenter | QLabel::AlignVCenter);

    lfader = new fader(sliderbox, channelnames[channel]);
    lfader->setValue(lvol);
    connect(lfader, SIGNAL(valueChanged(int)), SLOT(SetLVal(int)) );

    if(IsStereo){
        rfader = new fader(sliderbox, channelnames[channel]);
        rfader->setValue(rvol);
        connect(rfader, SIGNAL(valueChanged(int)), SLOT(SetRVal(int)) );
    }

    checkRec  = new QCheckBox("rec", box);
    checkMute = new QCheckBox("mute", box);
    checkLock = new QCheckBox("lock", box);

    connect(checkMute, SIGNAL(toggled(bool)), SLOT(enableMuting(bool)));

    if(!IsStereo)
        checkLock->setDisabled(1);
    else
        checkLock->toggle();

    if(IsRecDev){
        connect(checkRec, SIGNAL(stateChanged(int)), SLOT(SetRec(int)) );
        if(GetRec())
            checkRec->toggle();
    }
    else /* show button as disabled if this is no rec source */
        checkRec->setDisabled(1);
}


smixer :: smixer(char *filename){
    error = 0;      /* till now, we're ok! */
    /* open mixer: */

    fd = open(filename, O_RDWR);
    if(fd == -1){
        fprintf(stderr, "unable to open mixer device %s: ", filename);
        perror("");
        error = errno;
        return;
    }

    /* read all the mixer-infos */
    if(ioctl(fd, SOUND_MIXER_READ_DEVMASK , &devs) == -1){
        perror("mixer device failing (while reading devmask)");
        error = -1;
        return;
    }
    if(ioctl(fd, SOUND_MIXER_READ_RECMASK , &rec) == -1){
        perror("mixer device failing (while reading recmask)");
        error = -1;
        return;
    }
    if(ioctl(fd, SOUND_MIXER_READ_STEREODEVS , &st) == -1){
        perror("mixer device failing (while reading stereomask)");
        error = -1;
        return;
    }
    if(ioctl(fd, SOUND_MIXER_INFO , &info) == -1){
        perror("mixer device failing (while reading info)");
        error = -1;
        return;
    }


}

smixer :: ~smixer(){

    close(fd);
}

void mchannel :: SetLVal(int value){
    if(checkLock->isChecked()){
        lvol = rvol = value;
        value = value | value << 8;
    }
    else{
        value |= rvol << 8;
        lvol = value;
    }
    ioctl(mixer->fd, MIXER_WRITE(channel), &value);
    updateAll();
}

void mchannel :: SetRVal(int value){
    if(checkLock->isChecked()){
        lvol = rvol = value;
        value = value | value << 8;
    }
    else{
        value = lvol | value << 8;
        rvol = value;
    }
    ioctl(mixer->fd, MIXER_WRITE(channel), &value);
    updateAll();
}

int mchannel :: GetLVal(){
    int value;
    ioctl(mixer->fd, MIXER_READ(channel), &value);
    return value & 0xff;
}

int mchannel :: GetRVal(){
    int value;
    ioctl(mixer->fd, MIXER_READ(channel), &value);
    return (value >> 8) & 0xff;
}

void mchannel :: SetRec(int recval){
    int recstat;
    ioctl(mixer->fd, SOUND_MIXER_READ_RECSRC, &recstat);
    if(recval == 2)
        recstat |= (1 << channel);
    else if(recval == 0)
        recstat &= ~(1 << channel);

    ioctl(mixer->fd, SOUND_MIXER_WRITE_RECSRC, &recstat);
    emit recsChanged();
}

bool mchannel :: GetRec(){
    int recstat;
    ioctl(mixer->fd, SOUND_MIXER_READ_RECSRC, &recstat);
    if(recstat & (1<<channel))
        return 1;
    else
        return 0;
}

void mchannel :: updateAll(){
    int vols;
    updateRecs();
    ioctl(mixer->fd, MIXER_READ(channel), &vols);

    rvol = (vols >> 8) & 0xff;
    lvol = vols & 0xff;

    lfader->setValue(lvol);

    if(IsStereo)
        rfader->setValue(rvol);
}

void mchannel :: updateRecs(){
    int recstat;
    ioctl(mixer->fd, SOUND_MIXER_READ_RECSRC, &recstat);
    if(recstat & (1<<channel))
        checkRec->setChecked(1);
    else
        checkRec->setChecked(0);
}

/* enable(bool)
 * enable/disable display of this channel
 * and set entry in viewMask (for config file */
void mchannel :: enable(bool enable){
    if(enable){
        show();
//        emit(enableView(channel));
    }
    else{
        hide();
//        emit(disableView(channel));
    }
}

void mchannel :: enableMuting(bool enable){
    if(enable){
        lmute = lvol;
        rmute = rvol;
        SetLVal(0);
        SetRVal(0);
    }
    else{
        SetLVal(lmute);
        SetRVal(rmute);
    }

    lfader->setDisabled(enable);
    if(IsStereo)
        rfader->setDisabled(enable);
}


QString mchannel :: getName(){
    return channelLabel->text();
}

bool mchannel :: getMuteState(){
    return checkMute->isChecked();
}

bool mchannel :: getRecState(){
    return checkRec->isChecked();
}

bool mchannel :: getLockState(){
    return checkLock->isChecked();
}

int mchannel :: getLVolume(){
    return lvol;
}

int mchannel :: getRVolume(){
    return rvol;
}

int mchannel :: getLMuteVol(){
    return lmute;
}

int mchannel :: getRMuteVol(){
    return rmute;
}



void mchannel :: setLVolume( int val ){
    lfader->setValue( val );
}

void mchannel :: setRVolume( int val ){
    if(IsStereo)
        rfader->setValue( val );
}

void mchannel :: setLMuteVol( int val ){
    lmute = val;
}

void mchannel :: setRMuteVol( int val ){
    rmute = val;
}

void mchannel :: setMuteState( bool val ){
    checkMute->setChecked( val );
}

void mchannel :: setLockState( bool val ){
    checkLock->setChecked( val );
}

void mchannel :: setRecState( bool val ){
    checkRec->setChecked( val );
}

void mchannel :: setName( QString *name ){
    if(name){           /* name will be read as NULL from file if not set
                         * so dont change in this case */
        channelLabel->setText( *name );
    }
}



fader :: fader(QWidget *parent, const char *name) : QHBox(parent){
    setMinimumWidth(30);
    QVBox *vbox = new QVBox(this);
    disp = new QLabel(vbox);
    disp->setAlignment(QLabel::AlignHCenter | QLabel::AlignVCenter);
    slider = new QSlider(vbox, name);
    slider->setOrientation(QSlider::Vertical);
    slider->setMaxValue(100);

    connect(slider, SIGNAL(valueChanged(int)), this, SLOT(valueChanges(int)) );
}

void fader :: setValue(int value){
    slider->setValue(100 - value);
}

void fader :: valueChanges(int value){
    emit valueChanged(100 - value);
    disp->setNum(100 - value);
}

void printbin(int value){
    int i;
    for(i = 31; i >=0; i --){
        value & (1 << i) ? printf("1") : printf("0");
        if(! i % 8)
            printf(" ");
    }
    printf("\n");
}

int countones(int value){
    int i;
    int ones = 0;
    for(i = 0; i < SOUND_MIXER_NRDEVICES; i++)
        if(value & (1<<i))
            ones ++;
    return ones;
}


